/* Copyright (c) 2005, Jacklin Studios.  All rights reserved. */

var pitchNames = new Array(12);
pitchNames[0] = "C";
pitchNames[1] = "C#";
pitchNames[2] = "D";
pitchNames[3] = "D#";
pitchNames[4] = "E";
pitchNames[5] = "F";
pitchNames[6] = "F#";
pitchNames[7] = "G";
pitchNames[8] = "G#";
pitchNames[9] = "A";
pitchNames[10] = "A#";
pitchNames[11] = "B";

var altPitchNames = new Array(12);
altPitchNames[0] = null;
altPitchNames[1] = "Db";
altPitchNames[2] = null;
altPitchNames[3] = "Eb";
altPitchNames[4] = null;
altPitchNames[5] = null;
altPitchNames[6] = "Gb";
altPitchNames[7] = null;
altPitchNames[8] = "Ab";
altPitchNames[9] = null;
altPitchNames[10] = "Bb";
altPitchNames[11] = null;

var	tuningBase = 440.0;


function noteNumToFreq(noteNum) {
	return tuningBase * Math.pow(2, (noteNum - 69.0) / 12.0);
}

function freqToNoteNum(freq) {
	return Math.round((12.0 * (Math.log(freq / (tuningBase / 2.0)) / Math.log(2.0))) + 57.0);
}


var	lastOcvate;
var lastPitch = -1;

var	octaveMiddleC = 3;

function noteNumToNoteName(noteNum) {
	var	octaveAdjust = 0;
	while (noteNum < 0) {
		noteNum += 12;
		octaveAdjust -= 1;
	}
	var	pitch = noteNum % 12;
	var octave = (Math.floor(noteNum / 12) - (5 - octaveMiddleC)) + octaveAdjust;
	var noteName = pitchNames[pitch] + octave.toString();
	if (altPitchNames[pitch] != null) {
		noteName += "/" + altPitchNames[pitch] + octave.toString();
	}
	
	lastPitch = pitch;
	lastOctave = octave;
	
	return noteName;
}

var instrumentBasePitches = new Array();
instrumentBasePitches["bass"] = new Array(5);
instrumentBasePitches["bass"][0] = 43;
instrumentBasePitches["bass"][1] = 38;
instrumentBasePitches["bass"][2] = 33;
instrumentBasePitches["bass"][3] = 28;
instrumentBasePitches["bass"][4] = 23;
instrumentBasePitches["guitar"] = new Array(6);
instrumentBasePitches["guitar"][0] = 64;
instrumentBasePitches["guitar"][1] = 59;
instrumentBasePitches["guitar"][2] = 55;
instrumentBasePitches["guitar"][3] = 50;
instrumentBasePitches["guitar"][4] = 45;
instrumentBasePitches["guitar"][5] = 40;
instrumentBasePitches["guitar"][6] = 35;
instrumentBasePitches["mandolin"] = new Array(4);
instrumentBasePitches["mandolin"][0] = 76;
instrumentBasePitches["mandolin"][1] = 69;
instrumentBasePitches["mandolin"][2] = 62;
instrumentBasePitches["mandolin"][3] = 55;

var	lastInstrumentName = null;

function instrumentNoteNameToNoteNum(noteName) {
	try {
		var	breakAt = noteName.indexOf(":");
		if (breakAt != -1) {
			var	instrumentName = null;
			if (breakAt != 0) {
				instrumentName = noteName.substring(0, breakAt).toLowerCase();
			}
			else if (lastInstrumentName != null) {
				instrumentName = lastInstrumentName;
			}
			
			if (instrumentName && (instrumentName.length > 0)) {
				var	basePitches = instrumentBasePitches[instrumentName];
				if (basePitches && (basePitches.length > 0)) {
					lastInstrumentName = instrumentName;
					var	restStr = noteName.substring(breakAt + 1);
					var	index = parseInt(restStr);
					if ((index > 0) && (index <= basePitches.length)) {
						var	baseNote = basePitches[index - 1];
						breakAt = restStr.indexOf("+");
						if (breakAt > 0) {
							baseNote += parseInt(restStr.substring(breakAt));
						}
						return baseNote;
					}
				}
			}
		}
	} catch (e) {}
	return -1;
}

function noteNameToNoteNum(noteName) {
	var	instrumentMatch = instrumentNoteNameToNoteNum(noteName);
	if (instrumentMatch != -1) {
		return instrumentMatch;
	}
	
	var	octave = lastOctave;
	
	var	octaveIndex = noteName.search(/[-+]*\d+/);
	if (octaveIndex != -1) {
		var	octaveStr = noteName.substring(octaveIndex);
		octave = parseInt(octaveStr);
		if (isNaN(octave) || (octave < -4) || (octave > 11)) {
			octave = lastOctave;
		}
	}
	
	var	noteStr = noteName.charAt(0).toUpperCase();
	var	noteVariant = noteName.charAt(1);
	if ((noteVariant == "b") || (noteVariant == "#")) {
		noteStr += noteVariant;
	}

	var pitchMatch = null;
	var	pitchNum = -1;
	for (var i=11; i>=0; i--) {
		if ((altPitchNames[i] != null) && (noteStr.indexOf(altPitchNames[i]) == 0)) {
			pitchNum = i;
			pitchMatch = altPitchNames[i];
			break;
		}
	}
	if (pitchNum < 0) {
		for (var i=11; i>=0; i--) {
			if (noteStr.indexOf(pitchNames[i]) == 0) {
				pitchNum = i;
				pitchMatch = pitchNames[i];
				break;
			}
		}
	}
	if (pitchNum < 0) {
		pitchNum = lastPitch;
	}

	return Math.round(pitchNum + ((octave + (5 - octaveMiddleC)) * 12));
}

function setResults(midi, note, freq) {
	if (midi < 0) {
		midi = "" + midi + " (<0)"
	}
	else if (midi > 127) {
		midi = "" + midi + " (>127)"
	}
	document.getElementById("midi").value = midi;
	document.getElementById("note").value = note;
	
	var	temp = Math.round(freq * 1000);
	var	fixedDigits = 0;
	if ((temp % 10) != 0) {
		fixedDigits = 3;
	}
	else if ((temp % 100) != 0) {
		fixedDigits = 2;
	}
	else if ((temp % 1000) != 0) {
		fixedDigits = 1;
	}
	var	freqStr = freq.toFixed(fixedDigits);
	document.getElementById("freq").value = freqStr;
	
	if (window.widget) {
		widget.setPreferenceForKey(freqStr, widget.identifier + "-freq");
	}
}

function convertNote() {
	var	note = document.getElementById("note").value;
	var	midi = noteNameToNoteNum(note);
	var	freq = noteNumToFreq(midi);
	note = noteNumToNoteName(midi);
	setResults(midi, note, freq);
}

var	lastFreq = tuningBase;

function convertMidiValue(midi) {
	var	freq = noteNumToFreq(midi);
	var	note = noteNumToNoteName(midi);
	setResults(midi, note, freq);
	lastFreq = freq;
}

function convertMidi() {
	var	midiStr = document.getElementById("midi").value;
	var	midi = parseInt(midiStr);
	if (midiStr && (midiStr.length > 0) && (! isNaN(midi)) && (midi > -80) && (midi < 138)) {
		convertMidiValue(midi);
	}
	else {
		convertFreqValue(lastFreq);
	}
}

function convertFreqValue(freq) {
	var	midi = freqToNoteNum(freq);
	var	note = noteNumToNoteName(midi);
	freq = noteNumToFreq(midi);
	setResults(midi, note, freq);
	lastFreq = freq;
}

function convertFreq() {
	var	freqStr = document.getElementById("freq").value;
	if (freqStr && (freqStr.length > 0) && validateFreq(freqStr)) {
		convertFreqValue(parseFloat(freqStr));
	}
	else {
		convertFreqValue(lastFreq);
	}
}


function validateFloat(value, minValue, maxValue) {
	if ((! isNaN(value)) && (value > minValue) && (value < maxValue)) {
		return true;
	}
	return false;
}

function validateFloatStr(floatStr, minValue, maxValue) {
	try {
		if (floatStr && (floatStr.length > 0)) {
			return validateFloat(parseFloat(floatStr), minValue, maxValue);
		}
	} catch (e) {}
	return false;
}

function validateFreq(freqStr) {
	return validateFloatStr(freqStr, 0.0, 22000.0);
}

function validateTuningBase(tuneBaseStr) {
	return validateFloatStr(tuneBaseStr, 370.0, 480.0);
}


function setTuningBaseTo(value) {
	tuningBase = value;
	convertNote();
}

function setPrefsTuningBase() {
	var	tuningBaseStr = document.getElementById("tunebase").value;
	if (validateTuningBase(tuningBaseStr)) {
		setTuningBaseTo(parseFloat(tuningBaseStr));
		if (window.widget) {
			widget.setPreferenceForKey(tuningBaseStr, "tunebase");
		}
		document.getElementById("tunebase").value = "" + tuningBase + " Hz";
	}
	else {
		focusDefaultBack();
	}
}

function setOctaveMiddleCTo(value) {
	octaveMiddleC = value;
	convertNote();
}

function setPrefsOctaveMiddleC() {
	var	octaveStr = document.getElementById("middle-c").value;
	if (octaveStr && (octaveStr.length == 1)) {
		if (octaveStr == "4") {
			setOctaveMiddleCTo(4)
		}
		else {
			setOctaveMiddleCTo(3);
			octaveStr = null;
		}
		if (window.widget) {
			widget.setPreferenceForKey(octaveStr, "middle-octave");
		}
	}
}

function onremove() {
	if (window.widget) {
		widget.setPreferenceForKey(null, "tunebase");
		widget.setPreferenceForKey(null, widget.identifier + "-freq");
		widget.setPreferenceForKey(null, "middle-octave");
	}
}

function setup() {
	if (window.widget) {
		widget.onremove = onremove;
	}

	var	defaultFreq = "440";
	if (window.widget) {
		var prefsTuningBase = widget.preferenceForKey("tunebase");
		if (validateTuningBase(prefsTuningBase)) {
			tuningBase = parseFloat(prefsTuningBase);
			defaultFreq = tuningBase;
			document.getElementById("tunebase").value = prefsTuningBase;
		}
		
		var	prefsFreq = widget.preferenceForKey(widget.identifier + "-freq");
		if (validateFreq(prefsFreq)) {
			defaultFreq = tuningBase;
		}
		
		var	prefsMiddleOctave = widget.preferenceForKey("middle-octave");
		if (prefsMiddleOctave && (prefsMiddleOctave.length > 0) && (prefsMiddleOctave == "4")) {
			octaveMiddleC = 4;
			document.getElementById("middle-c").value = "4";
		}
	}
	
	document.getElementById("freq").value = "440";
	convertFreq();
	focusDefault();
}

function focusDefault() {
	document.getElementById("note").focus();
}

function focusDefaultBack() {
	document.getElementById("tunebase").focus();
}

